@@ -9,18 +9,26 @@ shinyServer(function(input, output, session) {
     } else input$plot_selected
   })
 
-  output$plot <- renderggiraph({
+  output$plot <- renderGirafe({
     p <- ggplot(aes(x=Sepal.Length,y=Petal.Length, data_id = Species ),data=iris) +
       geom_point_interactive(size = 3) + theme_minimal()
-    ggiraph(code = print(p),
-            hover_css = "fill:red;cursor:pointer;",
-            selection_type = "single",
-            selected_css = "fill:orange;", width = 1)
+    girafe(
+      ggobj = p,
+      options = list(
+        opts_hover(css = "fill:red;cursor:pointer;"),
+        opts_selection(type = "single", css = "fill:orange;")
+      )
+    )
   })
-
-  observe( {
+  output$selpoint <- renderUI({
     value <- selected_car()
-    updateTextInput(session = session, "selpoint", value = paste0(value, collapse = ",") )
+    if( !isTruthy(value) )
+      value <- "<none>"
+    tags$div(
+      tags$caption("Selected point is:"),
+      tags$strong(value)
+    )
+
   })
 
 })
